#include <sourcemod>
#include <sdktools>
#include <cstrike>
#include <sdkhooks>
#include <dhooks>
#include "knives/globals"
#include "knives/convars"
#include "knives/database"
#include "knives/commands"
#include "knives/hooks"
#include "knives/menu"
#pragma semicolon 1

public Plugin myinfo =
{
    name = "CS:GO Knives",
    author = "Neuro Toxin",
    description = "A highly refined knife selection plugin",
    version = "1.1.3"
};

public void OnPluginStart()
{
	if (!HookOnGiveNamedItem())
	{
		SetFailState("Unable to hook GiveNamedItem using DHooks");
		return;
	}
	
	CreateConVars();
	BuildKnivesMenu();
	RegisterConsoleCommands();
	HookEvents();
}

public void OnConfigsExecuted()
{
	ConnectToDatabase();
}

public void OnDatabaseConnected()
{
	// Late loading here is the best place
	for (int client = 1; client < MaxClients; client++)
	{
		if (!IsClientConnected(client))
			continue;
			
		if (IsFakeClient(client))
			continue;
			
		OnClientConnected(client);
		
		if (!IsClientInGame(client))
			continue;
			
		OnClientPutInServer(client);			
			
		if (!IsClientAuthorized(client))
			continue;
		
		OnClientPostAdminCheck(client);
	}
}

public void OnClientConnected(int client)
{
	g_iPlayerKnifeSelections[client] = 0;
	g_iPlayerKnifeKills[client] = 0;
}

public void OnClientPutInServer(int client)
{
	if (IsFakeClient(client))
		return;
	
	HookPlayer(client);
}

public void OnClientPostAdminCheck(int client)
{
	if (IsFakeClient(client))
		return;
		
	LoadPlayerData(client);
}

public void OnClientDisconnect(int client)
{
	if (IsFakeClient(client))
		return;
		
	UnhookPlayer(client);
}

/*
 * This is a prehook to change the classnames of the item being spawned if required.
 * -- This stops the animation glitch of some knives as the item is forced to spawn with the correct classtype
*/
public MRESReturn OnGiveNamedItem(int client, Handle hReturn, Handle hParams)
{
	// Skip if player hasnt selected a knife
	if (!g_iPlayerKnifeSelections[client])
		return MRES_Ignored;
	
	// Get classname and skip processing if it's not a weapon
	static char classname[64];
	DHookGetParamString(hParams, 1, classname, sizeof(classname));
	
	// -> not sure if leaving this this is a performance boost or not
	if (StrContains(classname, "weapon_") != 0)
		return MRES_Ignored;
	
	// Update the classname and if successful ...
	if (UpdateKnifeClassname(client, classname, sizeof(classname)))
	{
		PrintToConsole(client, "==============-----------------> OnGiveNamedItemReset(classname=%s)", classname);
		
		// Give the player the correct classnamed item
		int weapon = GivePlayerItem(client, classname);
		if (weapon > -1) EquipPlayerWeapon(client, weapon);
		else weapon = 0;
		
		// Return the new weapin index and block this call
		DHookSetReturn(hReturn, weapon);
		return MRES_Supercede;
	}
	
	PrintToConsole(client, "==============-----------------> OnGiveNamedItem(classname=%s)", classname);
	return MRES_Ignored;
}

public MRESReturn OnGiveNamedItemPost(int client, Handle hReturn, Handle hParams)
{
	// Skip if player hasnt selected a knife
	if (!g_iPlayerKnifeSelections[client])
		return MRES_Ignored;
	
	// Get classname and skip processing if it's not a weapon
	// -> this is done first as some named items dont contain netprop m_iItemDefinitionIndex
	static char classname[64];
	DHookGetParamString(hParams, 1, classname, sizeof(classname));
	if (StrContains(classname, "weapon_") != 0)
		return MRES_Ignored;
		
	// For some reason the orginal blocked hook appears to have its post called via dhooks
	// -> simply skip processing for the default knife classnames
	if (StrEqual(classname, "weapon_knife"))
		return MRES_Ignored;
	else if (StrEqual(classname, "weapon_knife_t"))
		return MRES_Ignored;
	
	int weapon = DHookGetReturn(hReturn);
	PrintToConsole(client, "===============---------------------> OnGiveNamedItemPost(classname=%s, entity=%d)", classname, weapon);
	
	// Get definition index
	int defindex = GetEntProp(weapon, Prop_Send, "m_iItemDefinitionIndex");
	
	// Skip if player is not getting a knife
	bool isknife = IsDefIndexKnife(defindex);
	if (!isknife)
		return MRES_Ignored;
		
	// Set item definition if server put it back to the players loadout definition
	if (g_iPlayerKnifeSelections[client] != defindex)
	{
		// Set knife definition index to players selection
		SetEntProp(weapon, Prop_Send, "m_iItemDefinitionIndex", g_iPlayerKnifeSelections[client]);
		PrintToConsole(client, "=================---------------------> forced defindex=%d", g_iPlayerKnifeSelections[client]);
	}
	
	if (cvar_knives_safemode)
		return MRES_Ignored;
		
	// Dont run the below in safemode
	if (cvar_knives_restore_itemid)
	{
		Handle pack = CreateDataPack();
		WritePackCell(pack, EntIndexToEntRef(weapon));
		WritePackCell(pack, GetEntProp(weapon, Prop_Send, "m_iItemIDLow"));
		ResetPack(pack);
		CreateTimer(0.1, OnResetWeaponItemIDLow, pack);
	}
	
	// Remove paintkit info
	SetEntProp(weapon, Prop_Send, "m_iEntityQuality", 3); // NOTE: Setting this for knives almost eliminated all client crashes
	SetEntProp(weapon, Prop_Send, "m_nFallbackPaintKit", 0);
	SetEntProp(weapon, Prop_Send, "m_iItemIDLow", -1);
	
	// Set stattrak kills
	SetEntProp(weapon, Prop_Send, "m_nFallbackStatTrak", g_iPlayerKnifeKills[client]);
	
	// I reset the seed and wear as they leak through from a players paintkit
	SetEntProp(weapon, Prop_Send, "m_nFallbackSeed", 100);
	SetEntPropFloat(weapon, Prop_Send, "m_flFallbackWear", 0.0001);
	
	return MRES_Ignored;
}

public Action OnResetWeaponItemIDLow(Handle timer, any pack)
{
	int weapon = EntRefToEntIndex(ReadPackCell(pack));
	
	if (weapon == -1)
	{
		CloseHandle(pack);
		return Plugin_Continue;
	}
	
	int itemidlow = ReadPackCell(pack);
	CloseHandle(pack);
	
	SetEntProp(weapon, Prop_Send, "m_iItemIDLow", itemidlow);
	return Plugin_Continue;
}

public Action OnTakeDamageAlive(int victim, int &attacker, int &inflictor, float &damage, int &damagetype, int &weapon, float damageForce[3], float damagePosition[3])
{
	if (float(GetClientHealth(victim)) - damage > 0.0)
		return Plugin_Continue;
		
	if (!(damagetype & DMG_SLASH))
		return Plugin_Continue;
		
	if (attacker < 1 || attacker > MaxClients)
		return Plugin_Continue;
		
	if (!IsClientInGame(attacker))
		return Plugin_Continue;

	new attackerteam = GetClientTeam(attacker);
	new victimteam = GetClientTeam(victim);
	
	if (attackerteam == victimteam)
		return Plugin_Continue;
		
	if (weapon < MAXPLAYERS)
		return Plugin_Continue;
		
	if (GetEntProp(weapon, Prop_Send, "m_nFallbackStatTrak") == -1)
		return Plugin_Continue;
	
	g_iPlayerKnifeKills[attacker]++;
	UpdatePlayerKnifeKills(attacker);
	return Plugin_Continue;
}

stock bool IsDefIndexKnife(int defindex)
{
	if (defindex == 42 || defindex == 59)
		return true;
		
	int count = sizeof(g_iKnifeDefinitions);
	for (int i = 0; i < count; i++)
	{
		if (g_iKnifeDefinitions[i] == defindex)
			return true;
	}
	return false;
}

public Action:OnRoundPostStart(Handle:event, const String:name[], bool:dontBroadcast)
{
	for (int client = 1; client < MAXPLAYERS; client++)
	{
		if (!IsClientInGame(client))
			continue;
			
		if (!g_iPlayerKnifeSelections[client])
			continue;
		
		if (!IsPlayerAlive(client))
			continue;
		
		int weapon = GetPlayerWeaponSlot(client, CS_SLOT_KNIFE);
		if (weapon > -1)
		{
			int kills = GetEntProp(weapon, Prop_Send, "m_nFallbackStatTrak");
			if (kills == -1)
				continue;
				
			if (kills == g_iPlayerKnifeKills[client])
				continue;
			
			int defindex = GetEntProp(weapon, Prop_Send, "m_iItemDefinitionIndex");
			char classname[64];
			if (!GetItemDefinitionDetails(defindex, _, _, classname, sizeof(classname)))
				continue;
			
			ReplacePlayerKnife(client, classname);
		}
	}
	return Plugin_Continue;
}

stock bool GetItemDefinitionDetails(int defindex, char[] name="", int namelength=0, char[] classname="", int classnamelength=0)
{
	if (defindex == 0)
	{
		Format(name, namelength, "Default");
		Format(classname, classnamelength, "weapon_knife");
		return true;
	}
	
	int count = sizeof(g_iKnifeDefinitions);
	for (int i = 0; i < count; i++)
	{
		if (g_iKnifeDefinitions[i] == defindex)
		{
			Format(name, namelength, "%s", g_sKnifeDefinitionNames[i]);
			Format(classname, classnamelength, "%s", g_sKnifeClassnames[i]);
			return true;
		}
	}
	return false;
}

stock bool UpdateKnifeClassname(int client, char[] classname, length)
{
	// Add these two manually so they are not displayed in the menu
	if (StrEqual(classname, "weapon_knife"))
		return GetItemDefinitionDetails(g_iPlayerKnifeSelections[client], _, _, classname, length);
	else if (StrEqual(classname, "weapon_knife_t"))
		return GetItemDefinitionDetails(g_iPlayerKnifeSelections[client], _, _, classname, length);
	else
	{
		int count = sizeof(g_sKnifeClassnames);
		for (int i = 0; i < count; i++)
		{
			if (!StrEqual(g_sKnifeClassnames[i], classname))
				continue;
				
			if (g_iPlayerKnifeSelections[client] != g_iKnifeDefinitions[i])
				return GetItemDefinitionDetails(g_iPlayerKnifeSelections[client], _, _, classname, length);
		}
	}
	return false;
}

stock void ReplacePlayerKnife(int client, char[] classname)
{
	// We must remove any tasers before we equip a new knife
	bool equiptaser = false;
	int weapon = GetPlayerWeaponSlot(client, CS_SLOT_KNIFE);
	while (weapon > -1)
	{
		if (GetEntProp(weapon, Prop_Send, "m_iItemDefinitionIndex") == 31)
			equiptaser = true;
		
		RemovePlayerItem(client, weapon);
		AcceptEntityInput(weapon, "Kill");
		weapon = GetPlayerWeaponSlot(client, CS_SLOT_KNIFE);
	}
	
	weapon = GivePlayerItem(client, classname);
	if (weapon > -1) EquipPlayerWeapon(client, weapon);
	if (equiptaser) GivePlayerItem(client, "weapon_taser");
}