Handle g_hDatabase = INVALID_HANDLE;
bool g_bDatabaseFullyConnected = false;

public void ConnectToDatabase()
{
	if (g_hDatabase != null)
	{
		OnDatabaseConnected();
		return;
	}
	
	SQL_TConnect(OnDatabaseEndConnect, cvar_knives_database_name);
}

public bool IsDatabaseConnected()
{
	return g_bDatabaseFullyConnected;
}

public void OnDatabaseEndConnect(Handle driver, Handle database, const char[] error, any data)
{
	if (database == null)
	{
		LogError("%s", error);
		SetFailState("Error connecting to database.");
		return;
	}
	
	g_hDatabase = database;
	new String:ident[8];
	SQL_GetDriverIdent(driver, ident, sizeof(ident));
	
	if (!StrEqual(ident, "mysql"))
	{
		DisconnectFromDatabase();
		LogError("You must use a MySQL database!");
		SetFailState("Error connecting to database.");
		return;
	}
	
	SQL_TQuery(g_hDatabase, OnDatabaseCreatedTable, 
		"CREATE TABLE IF NOT EXISTS `knives_selections` (`steamid` varchar(32) NOT NULL, `knife` int(11) NOT NULL DEFAULT '-1', `kills` bigint(20) NOT NULL DEFAULT '0', PRIMARY KEY (`steamid`), UNIQUE KEY `steamid_UNIQUE` (`steamid`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;");
}

public void OnDatabaseCreatedTable(Handle database, Handle query, const char[] error, any pack)
{
	if (query == null)
	{
		LogError("%s", error);
		SetFailState("Error checking/creating `knives_selections` table.");
		return;
	}
	
	OnDatabaseConnected();
}

public void DisconnectFromDatabase()
{
	if (g_hDatabase != null)
	{
		CloseHandle(g_hDatabase);
		g_hDatabase = null;
		g_bDatabaseFullyConnected = false;
	}
}

public void LoadPlayerData(int client)
{
	char query[512];
	char steamid[32];
	GetClientAuthId(client, AuthId_Steam2, steamid, sizeof(steamid));
	Format(query, sizeof(query), "SELECT * FROM `knives_selections` WHERE `steamid`='%s';", steamid);
	SQL_TQuery(g_hDatabase, OnDatabaseGotPlayerData, query, GetClientUserId(client));
}

public void OnDatabaseGotPlayerData(Handle database, Handle query, const char[] error, any userid)
{
	if (query == null)
	{
		LogError("%s", error);
		return;
	}
	
	int client = GetClientOfUserId(userid);
	if (client == 0)
		return;
		
	if (SQL_GetRowCount(query) == 0)
		return;
		
	if (!SQL_FetchRow(query))
		return;
		
	g_iPlayerKnifeSelections[client] = SQL_FetchInt(query, 1);
	g_iPlayerKnifeKills[client] = SQL_FetchInt(query, 2);
	PrintToChat(client, "\x01[SM] \x04Your knife data has loaded.");
	
	if (IsClientInGame(client) && IsPlayerAlive(client))
	{
		int team = GetClientTeam(client);
		if (team == CS_TEAM_CT || team == CS_TEAM_T)
		{
			char classname[64];
			if (!GetItemDefinitionDetails(g_iPlayerKnifeSelections[client], _, _, classname, sizeof(classname)))
				return;
			ReplacePlayerKnife(client, classname);
		}
	}
}

public void UpdatePlayerKnife(int client)
{
	char query[512];
	char steamid[32];
	GetClientAuthId(client, AuthId_Steam2, steamid, sizeof(steamid));
	Format(query, sizeof(query), "INSERT INTO `knives_selections` (`steamid`, `knife`) VALUES ('%s', %d) ON DUPLICATE KEY UPDATE `knife`= VALUES(`knife`);", steamid, g_iPlayerKnifeSelections[client]);
	SQL_TQuery(g_hDatabase, OnDatabaseUpdatedPlayerKnife, query, GetClientUserId(client));
}

public void OnDatabaseUpdatedPlayerKnife(Handle database, Handle query, const char[] error, any userid)
{
	if (query == null)
	{
		LogError("%s", error);
		return;
	}
	
	int client = GetClientOfUserId(userid);
	if (client == 0)
		return;
		
	PrintToChat(client, "\x01[SM] \x04Your knife data has been updated.");
}

public void UpdatePlayerKnifeKills(int client)
{
	char query[512];
	char steamid[32];
	GetClientAuthId(client, AuthId_Steam2, steamid, sizeof(steamid));
	Format(query, sizeof(query), "UPDATE `knives_selections` SET `kills`=%d WHERE `steamid`='%s';", g_iPlayerKnifeKills[client], steamid);
	SQL_TQuery(g_hDatabase, OnDatabaseUpdatedPlayerKnifeKills, query, GetClientUserId(client));
}

public void OnDatabaseUpdatedPlayerKnifeKills(Handle database, Handle query, const char[] error, any userid)
{
	if (query == null)
		LogError("%s", error);
}